﻿using Connected.Classes;
using Connected.Components;
using System.Collections.ObjectModel;

namespace Connected.Services;
public class StateManagerService
{
	public void SaveOrUpdateComponentState(Base Component)
	{
		var current_state = new State();
		var properties = Component.GetType().GetProperties();
		current_state.Id = Component.Id;
		current_state.ComponentType = Component.GetType();
		if (current_state.Values is null) current_state.Values = new Dictionary<string, object>();
		foreach (var property in properties)
		{
			var value = property.GetValue(Component);
			if (value is not null)
				current_state.Values.Add(property.Name, value);
		}
		if (SavedStates is null) SavedStates = new ObservableCollection<State>();
		SavedStates.Add(current_state);
	}

	public State? LoadComponentState(string ComponentId)
	{
		if (string.IsNullOrEmpty(ComponentId)) return null;
		if (SavedStates is not null)
		{
			var Component = SavedStates.Where(state => state.Id == ComponentId).FirstOrDefault();
			return Component;
		}
		return null;
	}

	public bool RemoveComponentState(string ComponentId)
	{
		bool result = false;
		try
		{
			if (SavedStates is not null)
			{
				return SavedStates.Remove(SavedStates.FirstOrDefault(state => state.Id == ComponentId));
			}
		} catch (Exception ex)
		{
			result = false;
		}
		return result;
	}

	public bool ClearComponentStates()
	{
		bool result = false;
		try
		{
			if (SavedStates is not null)
			{
				SavedStates.Clear();
				result = true;
			}
		} catch
		{
			result = false;
		}
		return result;
	}

	private ObservableCollection<State>? SavedStates { get; set; } = new();

}
